﻿using System;
using System.Collections.ObjectModel;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using Microsoft.Maps.MapControl;

namespace RouteService
{
    public partial class MainPage : UserControl
    {
        private MapLayer routeLayer;
        private Canvas contentPopup;
        private TextBlock contentPopupText;

        public MainPage()
        {
             InitializeComponent();
             routeLayer = MyMap.FindName("RouteLayer") as MapLayer;
             contentPopup = MyMap.FindName("ContentPopup") as Canvas;
             contentPopupText = MyMap.FindName("ContentPopupText") as TextBlock;
        }

        private PlatformServices.GeocodeServiceClient geocodeClient;
        private PlatformServices.GeocodeServiceClient GeocodeClient
        {
            get
            {
                if (null == geocodeClient)
                {
                    geocodeClient = new PlatformServices.GeocodeServiceClient("CustomBinding_IGeocodeService");
                    geocodeClient.GeocodeCompleted += new EventHandler<PlatformServices.GeocodeCompletedEventArgs>(client_GeocodeCompleted);
                }
                return geocodeClient;
            }
        }

        private PlatformServices.RouteServiceClient routeClient;
        private PlatformServices.RouteServiceClient RouteClient
        {
            get
            {
                if (null == routeClient)
                {
                    routeClient = new PlatformServices.RouteServiceClient("CustomBinding_IRouteService");
                    routeClient.CalculateRouteCompleted += new EventHandler<PlatformServices.CalculateRouteCompletedEventArgs>(client_RouteCompleted);
                }
                return routeClient;
            }
        }


        private void GeocodeAddress(string address, RoutingState state)
        {
            PlatformServices.GeocodeRequest request = new PlatformServices.GeocodeRequest();
            request.Culture = MyMap.Culture;
            request.Query = address;
            request.ExecutionOptions = new PlatformServices.ExecutionOptions();
            request.ExecutionOptions.SuppressFaults = true;
            request.Credentials = new Credentials();
            request.Credentials.ApplicationId = App.Current.Resources["MyCredentials"] as string;


            request.Options = new PlatformServices.GeocodeOptions();
            request.Options.Filters = new ObservableCollection<PlatformServices.FilterBase>();
            PlatformServices.ConfidenceFilter filter = new PlatformServices.ConfidenceFilter();
            filter.MinimumConfidence = PlatformServices.Confidence.High;
            request.Options.Filters.Add(filter);

            if (null != state.output)
            {
                state.output.Text = "<geocoding in progress>";
                state.output.Foreground = new SolidColorBrush(Colors.Black);
            }

            GeocodeClient.GeocodeAsync(request, state);
        }

        private object lockObject = new object();

        private void client_GeocodeCompleted(object sender, PlatformServices.GeocodeCompletedEventArgs e)
        {
            RoutingState state = e.UserState as RoutingState;
            PlatformServices.GeocodeResult result = null;
            string outString;

            try
            {
                if (e.Result.ResponseSummary.StatusCode != PlatformServices.ResponseStatusCode.Success)
                {
                    outString = "error geocoding ... status <" + e.Result.ResponseSummary.StatusCode.ToString() + ">";
                }
                else if (0 == e.Result.Results.Count)
                {
                    outString = "No result";
                }
                else
                {
                    result = e.Result.Results[0];
                    outString = result.DisplayName;
                }
            }
            catch (Exception)
            {
                outString = "Exception raised";
            }

            // Update UI with geocode result.
            if (null != state.output)
            {
                state.output.Text = outString;
            }

            if (null == result)
            {
                result = new PlatformServices.GeocodeResult();
            }

            // Update state object ... when all the results are set, call route.
            bool doneGeocoding;
            lock (lockObject)
            {
                state.results[state.locationNumber] = result;
                doneGeocoding = state.GeocodesComplete;
            }

            if (doneGeocoding && state.GeocodesSuccessful)
            {
                //Clear any existing routes
                ClearRoute();

                //Calculate the route
                CalculateRoute(state.results);
            }
        }

        private PlatformServices.Waypoint GeocodeResultToWaypoint(PlatformServices.GeocodeResult result)
        {
            PlatformServices.Waypoint waypoint = new PlatformServices.Waypoint();
            waypoint.Description = result.DisplayName;
            waypoint.Location = new Location();
            waypoint.Location.Latitude = result.Locations[0].Latitude;
            waypoint.Location.Longitude = result.Locations[0].Longitude;
            return waypoint;
        }

        private void CalculateRoute(PlatformServices.GeocodeResult[] locations)
        {
            PlatformServices.RouteRequest request = new PlatformServices.RouteRequest();
            request.Culture = MyMap.Culture;
            request.Waypoints = new ObservableCollection<PlatformServices.Waypoint>();
            foreach (PlatformServices.GeocodeResult result in locations)
            {
                request.Waypoints.Add(GeocodeResultToWaypoint(result));
            }

            request.ExecutionOptions = new PlatformServices.ExecutionOptions();
            request.ExecutionOptions.SuppressFaults = true;
            request.Credentials = new Credentials();
            request.Credentials.ApplicationId = App.Current.Resources["MyCredentials"] as string;

            request.Options = new PlatformServices.RouteOptions();
            request.Options.RoutePathType = PlatformServices.RoutePathType.Points;
            FromOutput.Text += " --> " + ToOutput.Text;
            ToOutput.Text += " ... <routing>";

            // Make asynchronous call to fetch the data ... pass state object.
            RouteClient.CalculateRouteAsync(request);
        }

        private void ClearRoute()
        {
            //Deregister events for the children layer items that had events
            foreach (UIElement child in routeLayer.Children)
            {
                Ellipse point = child as Ellipse;
                if (point != null)
                {
                    point.MouseEnter -= point_MouseEnter;
                    point.MouseLeave -= point_MouseLeave;
                }
            }

            //Clear any existing routes
            routeLayer.Children.Clear();
        }

        private void client_RouteCompleted(object sender, PlatformServices.CalculateRouteCompletedEventArgs e)
        {
            string outString;
            try
            {
                if (e.Result.ResponseSummary.StatusCode != PlatformServices.ResponseStatusCode.Success)
                {
                    outString = "error routing ... status <" + e.Result.ResponseSummary.StatusCode.ToString() + ">";
                }
                else if (0 == e.Result.Result.Legs.Count)
                {
                    outString = "Cannot find route";
                }
                else
                {
                    Color routeColor = Colors.Blue;
                    SolidColorBrush routeBrush = new SolidColorBrush(routeColor);
                    outString = "Found route ... coloring route";
                    ToOutput.Foreground = routeBrush;

                    //Add the Polyline for the route
                    MapPolyline routeLine = new MapPolyline();
                    routeLine.Locations = new LocationCollection();
                    routeLine.Stroke = routeBrush;
                    routeLine.Opacity = 0.65;
                    routeLine.StrokeThickness = 5.0;
                    foreach (Location loc in e.Result.Result.RoutePath.Points)
                    {
                        routeLine.Locations.Add(loc);
                    }
                    routeLayer.Children.Add(routeLine);


                    // Add the route points
                    foreach (PlatformServices.ItineraryItem itineraryItem in e.Result.Result.Legs[0].Itinerary)
                    {
                        Ellipse point = new Ellipse();
                        point.Width = 10;
                        point.Height = 10;
                        point.Fill = new SolidColorBrush(Colors.Red);
                        point.Opacity = 0.65;
                        MapLayer.SetPosition(point, itineraryItem.Location);
                        MapLayer.SetPositionOrigin(point, PositionOrigin.Center);
                        point.Tag = itineraryItem;
                        point.MouseEnter += new System.Windows.Input.MouseEventHandler(point_MouseEnter);
                        point.MouseLeave += new System.Windows.Input.MouseEventHandler(point_MouseLeave);

                        routeLayer.Children.Add(point);
                    }

                    //Set the map view
                    LocationRect rect = new LocationRect(routeLine.Locations[0], routeLine.Locations[routeLine.Locations.Count - 1]);
                    MyMap.SetView(rect);
                }
            }
            catch (Exception)
            {
                outString = "Exception raised routing";
            }

            ToOutput.Text = outString;

        }

        void point_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            //Show a popup with data about that point
            Ellipse pin = sender as Ellipse;
            if (pin != null)
            {
                PlatformServices.ItineraryItem itineraryItem = pin.Tag as PlatformServices.ItineraryItem;
                if (itineraryItem != null)
                {
                    MapLayer.SetPosition(contentPopup, itineraryItem.Location);
                    MapLayer.SetPositionOffset(contentPopup, new Point(15, -50));
                    
                    string contentString = itineraryItem.Text;
                    //Remove tags from the string
                    Regex regex = new Regex("<(.|\n)*?>");
                    contentString = regex.Replace(contentString, string.Empty);
                    contentPopupText.Text = contentString;
                    contentPopup.Visibility = Visibility.Visible;
                }
            }
        }

        void point_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            //Hide the popup
            Ellipse point = sender as Ellipse;
            contentPopup.Visibility = Visibility.Collapsed;
        }

        private void ContentPopup_MouseEnter(object sender, MouseEventArgs e)
        {
            //Show the popup
            contentPopup.Visibility = Visibility.Visible;
            Canvas.SetZIndex(contentPopup, 10);
        }

        private void ContentPopup_MouseLeave(object sender, MouseEventArgs e)
        {
            //Hide the popup
            contentPopup.Visibility = Visibility.Collapsed;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            bool errorFound = false;

            if (string.IsNullOrEmpty(ToInput.Text))
            {
                ToOutput.Text = "missing address";
                errorFound = true;
            }
            if (string.IsNullOrEmpty(FromInput.Text))
            {
                FromOutput.Text = "missing address";
                errorFound = true;
            }
            if (errorFound)
            {
                return;
            }

            // Geocode locations in parallel.
            PlatformServices.GeocodeResult[] results = new PlatformServices.GeocodeResult[2];
            // From location.
            RoutingState state0 = new RoutingState(results, 0, FromOutput);
            GeocodeAddress(FromInput.Text, state0);
            // To location.
            RoutingState state1 = new RoutingState(results, 1, ToOutput);
            GeocodeAddress(ToInput.Text, state1);
        }

        private void ToInput_GotFocus(object sender, RoutedEventArgs e)
        {
            ToInput.SelectAll();
        }

        private void FromInput_GotFocus(object sender, RoutedEventArgs e)
        {
            FromInput.SelectAll();
        }
    }


    internal class RoutingState
    {
        internal RoutingState(PlatformServices.GeocodeResult[] resultArray, int index, TextBlock tb)
        {
            results = resultArray;
            locationNumber = index;
            output = tb;
        }

        internal bool GeocodesComplete
        {
            get
            {
                for (int idx = 0; idx < results.Length; idx++)
                {
                    if (null == results[idx])
                        return false;
                }
                return true;
            }
        }

        internal bool GeocodesSuccessful
        {
            get
            {
                for (int idx = 0; idx < results.Length; idx++)
                {
                    if (null == results[idx] || null == results[idx].Locations || 0 == results[idx].Locations.Count)
                    {
                        return false;
                    }
                }
                return true;
            }
        }

        internal PlatformServices.GeocodeResult[] results;
        internal int locationNumber;
        internal TextBlock output;
    }


  }
