﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Globalization;
using Microsoft.Maps.MapControl;

namespace SearchService
{
    public partial class MainPage : UserControl
    {
        private Dictionary<string, PlatformServices.SearchResultBase> searchResults = new Dictionary<string, PlatformServices.SearchResultBase>();
        private Color[] colors = { Colors.Blue, Colors.Red, Colors.Green, Colors.Orange, Colors.Purple };
        private int colorIndex = -1;

        public MainPage()
        {
            InitializeComponent();
        }

        private void Find_Button_Click(object sender, RoutedEventArgs e)
        {
            string query = Input.Text;
            if (!string.IsNullOrEmpty(query))
            {
                ExecuteSearch(query);
            }
        }

        private void ClearSearch()
        {
            // Deregister events for the children pin items that had events
            foreach (UIElement pin in SearchResultLayer.Children)
            {
                pin.MouseEnter -= Pushpin_MouseEnter;
                pin.MouseLeave -= Pushpin_MouseLeave;
            }

            SearchResultLayer.Children.Clear();
            searchResults.Clear();
        }

        private void ExecuteSearch(string query)
        {
            //Set up the search request
            PlatformServices.SearchServiceClient searchClient;

            //Create the Service Client
            searchClient = new PlatformServices.SearchServiceClient("BasicHttpBinding_ISearchService");
            searchClient.SearchCompleted += new EventHandler<PlatformServices.SearchCompletedEventArgs>(searchClient_SearchCompleted);

            PlatformServices.SearchRequest request = new PlatformServices.SearchRequest();
            request.Culture = MyMap.Culture;
            request.Query = query;
            request.ExecutionOptions = new PlatformServices.ExecutionOptions();
            request.ExecutionOptions.SuppressFaults = true;
            request.Credentials = new Credentials();
            request.Credentials.ApplicationId = App.Current.Resources["MyCredentials"] as string;
            request.UserProfile = new PlatformServices.UserProfile();
            request.UserProfile.MapView = MyMap.BoundingRectangle;

            //execute the request
            searchClient.SearchAsync(request);
        }

        private void searchClient_SearchCompleted(object sender, PlatformServices.SearchCompletedEventArgs e)
        {
            string outputString;
            try
            {
                if (e.Result.ResponseSummary.StatusCode == PlatformServices.ResponseStatusCode.Success)
                {
                    outputString = string.Format(CultureInfo.InvariantCulture, "Found {0} results", e.Result.ResultSets[0].Results.Count);
                    IList<Location> locations = new List<Location>();

                    if ((e.Result != null) && (e.Result.ResultSets.Count > 0))
                    {
                        colorIndex = (colorIndex + 1) % colors.Length;
                        Color pinColor = colors[colorIndex];

                        int pushpinNumber = 0;
                        foreach (PlatformServices.SearchResultBase result in e.Result.ResultSets[0].Results)
                        {
                            ++pushpinNumber;
                            searchResults.Add(result.Id, result);

                            if (result.LocationData.Locations.Count > 0)
                            {
                                Location location = result.LocationData.Locations[0];
                                //Create a pushpin
                                Pushpin pushpin = new Pushpin();
                                pushpin.Location = location;
                                pushpin.Tag = result.Id;
                                pushpin.Background = new SolidColorBrush(pinColor);
                                pushpin.Content = pushpinNumber;

                                SearchResultLayer.Children.Add(pushpin);
                                locations.Add(location);

                                //Set up events for the pushpin
                                pushpin.MouseEnter += new MouseEventHandler(Pushpin_MouseEnter);
                                pushpin.MouseLeave += new MouseEventHandler(Pushpin_MouseLeave);
                            }
                        }

                        if (locations.Count > 0)
                        {
                            //If Search results were found, use the bounding area of the results
                            MyMap.SetView(new LocationRect(locations));
                        }
                        else
                        {
                            //Otherwise, if the SearchRegion is set from the service response, go ahead use that
                            LocationRect boundingArea = e.Result.ResultSets[0].SearchRegion.BoundingArea as LocationRect;
                            if (boundingArea != null)
                            {
                                MyMap.SetView(boundingArea);
                            }
                        }
                    }
                }
                else
                {
                    outputString = "error searching... status <" + e.Result.ResponseSummary.StatusCode.ToString() + ">";
                }
            }
            catch (Exception)
            {
                outputString = "Exception Raised";
            }

            Output.Text = outputString;
        }

        void Pushpin_MouseLeave(object sender, MouseEventArgs e)
        {
            //Hide the popup
            UIElement content = sender as UIElement;
            Canvas.SetZIndex(content, 100);
            ContentPopup.Visibility = Visibility.Collapsed;
        }

        void Pushpin_MouseEnter(object sender, MouseEventArgs e)
        {
            //Show the popup
            Pushpin pushpin = sender as Pushpin;
            Canvas.SetZIndex(pushpin, 500);
            Point point = e.GetPosition(MyMap);
            string resultId = pushpin.Tag as string;
            PlatformServices.SearchResultBase searchResult = searchResults[resultId];
            MapLayer.SetPosition(ContentPopup, searchResult.LocationData.Locations[0]);
            MapLayer.SetPositionOffset(ContentPopup, new Point(25, -50));

            PlatformServices.SearchResultBase result = searchResults[resultId];
            if (result != null)
            {
                ContentPopupText.Text = result.Name;
                PlatformServices.BusinessSearchResult businessResult = result as PlatformServices.BusinessSearchResult;
                if (businessResult != null)
                {
                    ContentPopupDescription.Text = string.Format("{0}\n{1}\n", businessResult.Address.FormattedAddress, businessResult.PhoneNumber);
                }
                ContentPopup.Visibility = Visibility.Visible;
            }
        }

        private void ContentPopup_MouseEnter(object sender, MouseEventArgs e)
        {
            //Show the popup if mouse is hovering over it
            ContentPopup.Visibility = Visibility.Visible;
            Canvas.SetZIndex(ContentPopup, 10);
        }

        private void ContentPopup_MouseLeave(object sender, MouseEventArgs e)
        {
            //Hide the popup if mouse leaves it
            ContentPopup.Visibility = Visibility.Collapsed;
        }

        private void Clear_Button_Click(object sender, RoutedEventArgs e)
        {
            //Clear any existing searches
            ClearSearch();
        }
    }
}
